// SPDX-FileCopyrightText: 2021 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

#login {
  background-image: none;

  .login-form {
    background: none;
    box-shadow: none;
  }
}

.orb {
  display: block;
  position: absolute;
  pointer-events: none;
  transform-origin: center center;
}

.orb img {
  height: auto;
  width: 100%;
}

#orb-analytics {
  top: -19%;
  width: 25%;
  left: 22.5%;
  z-index: 0;
}

#orb-analytics img {
  opacity: .2;
}

#orb-automation {
  bottom: -6%;
  width: 60%;
  left: 7%;
  z-index: 0;
  margin-left: -30%;
  margin-bottom: -30%;
}

#orb-automation img {
  opacity: .75;
}

#orb-cloud {
  top: -6%;
  width: 25%;
  right: 4%;
  z-index: 0;
  margin-right: -12.5%;
  margin-top: -12.5%;
}

#orb-cloud img {
  opacity: .4;
}

#orb-notifactions {
  top: 7%;
  right: 46%;
  width: 10%;
  margin: -5%;
}

#orb-notifactions img {
  opacity: .5;
}

#orb-metrics {
  left: 5%;
  top: 20%;
  width: 35%;
  margin: -17.5%;
}

#orb-metrics img {
  opacity: .5;
}

#orb-icinga {
  left: 50%;
  top: 50%;
  margin-top: -38.5em;
  margin-left: -38em;
  width: 75em;
  z-index: 0;
}

#orb-icinga img {
  opacity: .8;
}

#orb-infrastructure {
  top: -36%;
  left: -15%;
  width: 30%;
}

#orb-infrastructure img {
  opacity: .6;
}
