<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Exception\Json;

/**
 * Exception thrown by {@link \Icinga\Util\Json::encode()} on failure
 */
class JsonEncodeException extends JsonException
{
}
