<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Doc\Exception;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if an error in the documentation module's library occurs
 */
class DocException extends IcingaException
{
}
