// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

#fontsize-calc {
  display: none;
  width: 1000em;
  height: 1em;
  font-size: 1em;
  position: absolute;
  top: -2em;
}
