<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Protocol\Ldap;

use Icinga\Exception\IcingaException;

/**
 * Class LdapException
 * @package Icinga\Protocol\Ldap
 */
class LdapException extends IcingaException
{
}
