<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Module\Doc\Exception;

/**
 * Exception thrown if a chapter was not found
 */
class ChapterNotFoundException extends DocException
{
}
